import { defineStore } from 'pinia'
import { computed, MaybeRefOrGetter, toRef } from 'vue'
import { steeringAuthInit } from '@/stores/auth'
import {
  APIListUnpaginated,
  APIRetrieve,
  createExtendableAPI,
  createSteeringURL,
  ID,
  useAPIObject,
} from '@/api'
import { License } from '@/types'
import { useI18n } from '@/i18n'

type LicensedWork = {
  credits: string
  isUseExplicitlyGrantedByAuthor: boolean
}

export function useSelectedLicense(licenseId: MaybeRefOrGetter<ID | null>) {
  const store = useLicenseStore()
  const { obj } = useAPIObject(store, licenseId)
  return obj
}

export function useLicenseValidator(
  license: MaybeRefOrGetter<License | null>,
  licensedWork: MaybeRefOrGetter<LicensedWork>,
) {
  const _license = toRef(license)
  const _licensedWork = toRef(licensedWork)
  const { t } = useI18n()
  const isUseExplicitlyGrantedByAuthorError = computed(() => {
    if (
      _license?.value?.requiresExpressPermissionForPublication &&
      !_licensedWork.value.isUseExplicitlyGrantedByAuthor
    ) {
      return { message: t('license.validation.requiresExpressPermissionForPublication') }
    }

    return undefined
  })
  const needsAuthorError = computed(() => {
    if (_license?.value?.needsAuthor && !_licensedWork.value.credits.trim()) {
      return { message: t('license.validation.requiresCredits') }
    }
    return undefined
  })
  const hasError = computed(
    () => isUseExplicitlyGrantedByAuthorError.value || needsAuthorError.value,
  )
  return { isUseExplicitlyGrantedByAuthorError, needsAuthorError, hasError }
}

export const useLicenseStore = defineStore('licenses', () => {
  const endpoint = createSteeringURL.prefix('licenses')
  const { base, ...api } = createExtendableAPI<License>(endpoint, steeringAuthInit)
  return {
    ...base,
    ...APIListUnpaginated(api),
    ...APIRetrieve(api),
  }
})
